﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               Aero Glass                                 |
//|                              Version 1.00                                |
//|                                                                          |
//|        This project displays PNG sprites using variable opacity.         |
//|          Use either mouse or keyboard to move sprites arround.           |
//|                                                                          |
//|          The GDImage.dll unregistered version has no limitation,         |
//|             and all features of the API are fully functional.            |
//|                                                                          |
//|       Buying a commercial license removes the copyright nag screen.      |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 09-17-2006 (MM-DD-YYYY)            |
//|                        Last revised : 09-25-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;
//using System.Runtime;
//using System.IO;
//using Microsoft.Win32;

using Win32;
using GDImage;

namespace AeroGlass
{
    public delegate bool CallBack(IntPtr hWnd, uint wMsg, uint wParam, uint lParam);

    public partial class MAIN_Form : Form
    {
        const string Times_New_Roman = "Times New Roman"; // Must be a valid TTF
        const int ID_OBJECT_TEXT     = 9;                 // Identifiant unique pour le sprite text
	    const int ID_FIRST			 = ID_OBJECT_TEXT;
	    const int ID_OBJECT_SPRITE	 = 10;
	    const int ID_AERO_GLASS		 = 11;
	    const int ID_LIGHT			 = 12;
	    const int ID_TWIN		     = 13;
	    const int ID_BRUSH			 = 14;
	    const int ID_LAST			 = ID_BRUSH;

        const int ID_CTRL            = 100;               // GDImage Zoom control
        const int ID_STATUS_Main     = 102;
        const int ID_BTN_Fit         = 103;
        const int ID_CHECK_Glue      = 104;

        IntPtr[] SpriteBitmap = new IntPtr[4];

        static public IntPtr hFORM_Main = IntPtr.Zero;         // Default to Zero
        static public Boolean CHECK_Glue_Checked = false;
        
        // Very important the call back must be static
        // to prevent from access violation during garbage collection.
        // Search for <<Callback Functions>>
        static CallBack UseCallBack = null;

        public MAIN_Form()
        {
            // We use a callback to handle the GDImage events         
            // Very important the call back must be static
            // to prevent from access violation during garbage collection.
            // Search for <<Callback Functions>>
            UseCallBack = new CallBack(MAIN_Form.GDImageCallBack);

            InitializeComponent();

            // Create GDImage control
            if (GI.RegisterGDImageClass())
            {
                // Template(s) must be hidden and disabled
                // they are used to anchor the GDImage controls in MAIN_Form_Resize.
                GDImage_Ctrl_Template.Visible = false;
                GDImage_Ctrl_Template.Enabled = false;

                // Create the GDImage Zoom control
                IntPtr hCtrl = GDImage.CreateControl(GDImage_Ctrl_Template, ID_CTRL);               

                // We use a callback to monitor the GDImage control messages
                // Create a %WM_LBUTTONDOWN event
                GDImage.ZI_EventMessage(UseCallBack, Api.WM_LBUTTONDOWN, true);
                // Create a WM_RBUTTONDOWN event
                GDImage.ZI_EventMessage(UseCallBack, Api.WM_RBUTTONDOWN, true);
                // Create a WM_KEYDOWN event
                GDImage.ZI_EventMessage(UseCallBack, Api.WM_KEYDOWN, true);
                // Create a WM_MOUSEMOVE event
                GDImage.ZI_EventMessage(UseCallBack, Api.WM_MOUSEMOVE, true);

                GI.CaptureDesktop(hCtrl);     // Perform Screen shot (used as background)
                AddSpriteToCtrl(hCtrl);            // and add the sprites over it.

                Api.SetWindowLong(CHECK_Glue.Handle, Api.GWL_ID, ID_CHECK_Glue);
                Api.SetWindowLong(BTN_Fit.Handle, Api.GWL_ID, ID_BTN_Fit);
                Api.SetWindowLong(STATUS_Main.Handle, Api.GWL_ID, ID_STATUS_Main);

                hFORM_Main = this.Handle;          // Save the FORM_Main handle

            }

        }

        private void AddSpriteToCtrl(IntPtr hCtrl)
        {
            int i, x, y, bmW, bmH;
            uint ColorARGB = 0xFFFFFFFF;

            string ExePath = Application.StartupPath.TrimEnd('\\') + @"\";

            SpriteBitmap[0] = GI.ZI_CreateBitmapFromFile(ExePath + @"aero.png", out bmW, out bmH);
            SpriteBitmap[1] = GI.ZI_CreateBitmapFromFile(ExePath + @"light.png", out bmW, out bmH);
            SpriteBitmap[2] = GI.ZI_CreateBitmapFromFile(ExePath + @"twin.png", out bmW, out bmH);
            SpriteBitmap[3] = GI.ZI_CreateBitmapFromFile(ExePath + @"paintbrush.png", out bmW, out bmH);

            Random rnd = new Random();

            i = 0;
            for (int k = 0; k < SpriteBitmap.Length; k++)
            {
                i = k + 1;
                x = rnd.Next(0, GDImage_Ctrl_Template.Size.Width);
                y = rnd.Next(0, GDImage_Ctrl_Template.Size.Height);
                GI.ZD_DrawBitmapToCtrl(hCtrl, x, y, SpriteBitmap[k], ColorARGB, ID_OBJECT_SPRITE + i, GI.ZS_VISIBLE);
            }

            GI.ZD_SetObjectImageLabel(ID_AERO_GLASS, "Aero glass");
            GI.ZD_SetObjectScroll(ID_AERO_GLASS, true);
            GI.ZD_UsePngOpacity(ID_AERO_GLASS, true);

            GI.ZD_SetObjectImageLabel(ID_LIGHT, "Light");
            GI.ZD_SetObjectScroll(ID_LIGHT, true);
            GI.ZD_UsePngOpacity(ID_LIGHT, true);

            GI.ZD_SetObjectImageLabel(ID_TWIN, "Twin");
            GI.ZD_SetObjectScroll(ID_TWIN, true);
            GI.ZD_UsePngOpacity(ID_TWIN, true);

            GI.ZD_SetObjectImageLabel(ID_BRUSH, "Paint brush");
            GI.ZD_SetObjectScroll(ID_BRUSH, true);
            GI.ZD_UsePngOpacity(ID_BRUSH, true);

            // Draw overlayed text in a GDImage Control
            // ******************************************************************************
            // Require the use of True Type Font name (TTF).
            // This type of overlay doesn//t alter the image shown in the background.
            // ------------------------------------------------------------------------------
            // Draw sprite Text overlay
            // Note: because this is the last object it is drawn on top of z-order
            uint ARGBcolor = GI.ZD_ColorARGB(200, Api.RGB(250, 250, 255));
            GI.ZD_DrawTextToCtrl(
                    hCtrl,                // The GDImage control handle
                    "GDImage Aero Glass", // The text to display
                    20,                   // X coordinate
                    20,                   // Y coordinate
                    ARGBcolor,            // ARGB color to use
                    Times_New_Roman,      // True Type Font to use
                    40,                   // Font size in pixel
                    ID_OBJECT_TEXT,       // Unique object ID
                    GI.ZS_VISIBLE,        // Show 
                    1,                    // Shadow effect (offset in pixel)
                    0);                   // Orientation
            GI.ZD_SetObjectScroll(ID_OBJECT_TEXT, true);

        }

        // ****************************************
        // Callback that handles the GDImage events
        // Search for <<Callback Functions>>
        public static bool GDImageCallBack(IntPtr hWnd, uint wMsg, uint wParam, uint lParam)
        {
            Boolean bRet = false; // Do not stop the event processing in GDImage
            string LabelIs, sMessage;
            int ObjectID = 0;
            int x, y, x1, y1, useWidth, useHeight, BoundWidth, BoundHeight, 
                x2Div2, y2Div2, UseStep;
            Api.RECT rc = new Api.RECT();

            if (hWnd == Api.GetDlgItem(hFORM_Main, ID_CTRL))
            {
                switch (wMsg)
                {
                    case Api.WM_LBUTTONDOWN:
                        ObjectID = GI.ZI_MouseOverObjectID();
                        if (ObjectID == Api.GetDlgCtrlID(hWnd))
                        {
                            LabelIs = "Background";
                        }
                        else
                        {
                            LabelIs = GI.ZD_GetObjectImageLabel(ObjectID);
                        }
                        sMessage = @"WM_LBUTTONDOWN on object" + ObjectID.ToString() + 
                                   " >" + LabelIs + "<";
                        Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                        break;

                    case Api.WM_RBUTTONDOWN:
                        ObjectID = GI.ZI_MouseOverObjectID();
                        if (ObjectID == Api.GetDlgCtrlID(hWnd))
                        {
                            LabelIs = "Background";
                        }
                        else
                        {
                            LabelIs = GI.ZD_GetObjectImageLabel(ObjectID);
                        }
                        sMessage = @"WM_RBUTTONDOWN on object" + ObjectID.ToString() + 
                                   " >" + LabelIs + "< at location " + 
                                   Api.LoWrd(lParam).ToString() + "," + 
                                   Api.HiWrd(lParam).ToString();
                        Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                        break;

                    case Api.WM_MOUSEMOVE:
                        ObjectID = GI.ZI_GetMovingSpriteID();
                        if (ObjectID != 0)
                        {
                            if (CHECK_Glue_Checked) // Shall we move all sprites together ?
                            {
                                x = Api.LoWrd(lParam);
                                y = Api.HiWrd(lParam);
                                int xCurrentScroll = GI.ZI_GetProperty(hWnd, GI.ZI_Horizontal);
                                int yCurrentScroll = GI.ZI_GetProperty(hWnd, GI.ZI_Vertical);
                                int xCapture, yCapture, DX, DY;
                                GI.ZD_GetObjectXY(ObjectID, out x1, out y1);
                                GI.ZD_GetObjectXYcapture(ObjectID, out xCapture, out yCapture);
                                DX = (x - xCapture + xCurrentScroll) - x1;
                                DY = (y - yCapture + yCurrentScroll) - y1;

                                for (int ID = ID_FIRST; ID <= ID_LAST; ID++)
                                {
                                    if (ID != ObjectID)
                                    {
                                        GI.ZD_GetObjectXY(ID, out x, out y);
                                        GI.ZD_SetObjectXY(
                                            ID, x + DX, y + DY,   // Add DX,DY offset
                                            GI.ZD_DRAW_DEFERRED); // move without immediate redraw
                                    }
                                }
                            }
                        }
                        break;

                    case Api.WM_KEYDOWN:
                        ObjectID = GI.ZI_GetObjectFocusID();
                        if (ObjectID != 0)
                        {
                            GI.ZD_GetObjectXY(ObjectID, out x, out y);
                            x1 = x; // Make a copy to keep the orignal x location unchanged
                            y1 = y; // Make a copy to keep the orignal y location unchanged 

                            // Check accelerator keys to compute the step range 
                            if (GI.ZI_IsCtrlKeyPressed())
                            {
                                UseStep = 4;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 16;
                            }
                            else
                            {
                                UseStep = 1;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 2;
                            }

                            // If object scroll with the bitmap background
                            if (GI.ZD_GetObjectScroll(ObjectID))
                            {
                                // Get the size of the bitmap background
                                GI.ZI_GetBitmapSize(GI.ZI_GetBMP(Api.GetDlgItem(hFORM_Main, ID_CTRL)), out useWidth, out useHeight);
                            }
                            else
                            {
                                // Get the control client size  
                                Api.GetClientRect(Api.GetDlgItem(hFORM_Main, ID_CTRL), ref rc);
                                useWidth = rc.right; useHeight = rc.bottom;
                            }
                            // Get the sprite object size
                            GI.ZD_GetObjectBound(ObjectID, out BoundWidth, out BoundHeight);

                            x2Div2 = (int)(BoundWidth / 2); y2Div2 = (int)BoundHeight / 2;

                            switch (wParam)
                            {
                                case Api.VK_HOME: 
                                    x1 = 0;
                                    break;
                                case Api.VK_END:
                                    x1 = Math.Max(useWidth - BoundWidth, 0);
                                    break;
                                case Api.VK_PRIOR:
                                    y1 = 0;
                                    break;
                                case Api.VK_NEXT:
                                    y1 = Math.Max(useHeight - BoundHeight, 0);
                                    break;
                                case Api.VK_NUMPAD4:
                                case Api.VK_LEFT:
                                    if (x1 > -x2Div2) x1 = Math.Max(x1 - UseStep, -x2Div2);
                                    break;
                                case Api.VK_NUMPAD8:
                                case Api.VK_UP:
                                    if (y1 > -y2Div2) y1 = Math.Max(y1 - UseStep, -y2Div2);
                                    break;
                                case Api.VK_NUMPAD6:
                                case Api.VK_RIGHT:
                                    if (x1 < useWidth - x2Div2)
                                    {
                                        x1 = Math.Min(x1 + UseStep, useWidth - x2Div2);
                                    }
                                    break;
                                case Api.VK_NUMPAD2:
                                case Api.VK_DOWN:
                                    if (y1 < useHeight - y2Div2)
                                    {
                                        y1 = Math.Min(y1 + UseStep, useHeight - y2Div2);
                                    }
                                    break;
                                default:
                                    break;
                            }

                            if ((x != x1) || (y != y1))
                            {
                                x = x1; y = y1;
                                GI.ZD_SetObjectXY(ObjectID, x1, y1, GI.ZD_DRAW_REDRAW);
                            }
                            sMessage = "Object " + ObjectID.ToString() + " coordinates " + x.ToString() + "," + y.ToString();
                            Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                            bRet = true;
                        }
                        break;
                }
            }
            return bRet;
        }

        // ****************************************
        private void FORM_Main_Close(object sender, EventArgs e)
        {
            // Delete ALL sprite bitmaps
            for (int k = 0; k < SpriteBitmap.Length; k++)
            {
                if (SpriteBitmap[k] != IntPtr.Zero) Api.DeleteObject(SpriteBitmap[k]);
                SpriteBitmap[k] = System.IntPtr.Zero;  // Reset to Zero
            }
        }

        //***********************************************************
        // We use the .NET template to resize the GDImage control
        private void FORM_Main_Resize(object sender, EventArgs e)
        {
            IntPtr hCtrl = GDImage.Ctrl(this.Handle, ID_CTRL);
            Api.MoveWindow(hCtrl, GDImage_Ctrl_Template.Location.X, GDImage_Ctrl_Template.Location.Y, GDImage_Ctrl_Template.Size.Width, GDImage_Ctrl_Template.Size.Height, 1);
        }

        private void CHECK_Glue_CheckedChanged(object sender, EventArgs e)
        {
            if (CHECK_Glue.Checked) {
                CHECK_Glue_Checked = true;
            }
            else {
                CHECK_Glue_Checked = false;
            }
        }

        private void BTN_Fit_Click(object sender, EventArgs e)
        {
            int x, y, xCurrentScroll, yCurrentScroll, xGlass, yGlass, BoundWidthGlass, BoundHeightGlass,
            BoundWidth, BoundHeight;

            IntPtr hCtrl = Api.GetDlgItem(hFORM_Main, ID_CTRL);
            Api.RECT rc = new Api.RECT();
            Api.GetClientRect(hCtrl, ref rc);
//            Rectangle rc = FromHandle(hCtrl).RectangleToClient;

            xCurrentScroll = GI.ZI_GetProperty(hCtrl, GI.ZI_Horizontal);
            yCurrentScroll = GI.ZI_GetProperty(hCtrl, GI.ZI_Vertical);

            // ID_AERO_GLASS
            GI.ZD_GetObjectBound(ID_AERO_GLASS, out BoundWidthGlass, out BoundHeightGlass);
            xGlass = Math.Max((rc.right - BoundWidthGlass) / 2 + xCurrentScroll, 0);
            
            yGlass = Math.Max((rc.bottom - BoundHeightGlass) / 2 + yCurrentScroll, 0);
            GI.ZD_SetObjectXY(ID_AERO_GLASS, xGlass, yGlass, GI.ZD_DRAW_DEFERRED);

            // ID_OBJECT_TEXT
            GI.ZD_GetObjectBound(ID_OBJECT_TEXT, out BoundWidth, out BoundHeight);
            x = xGlass + Math.Max((BoundWidthGlass - BoundWidth) / 2, 0);
            y = yGlass + 10;
            GI.ZD_SetObjectXY(ID_OBJECT_TEXT, x, y, GI.ZD_DRAW_DEFERRED);

            // ID_LIGHT
            GI.ZD_GetObjectBound(ID_LIGHT, out BoundWidth, out BoundHeight);
            x = xGlass + BoundWidthGlass - (int)(BoundWidth * 0.56);
            y = yGlass - (BoundHeight / 3);
            GI.ZD_SetObjectXY(ID_LIGHT, x, y, GI.ZD_DRAW_DEFERRED);

            // ID_TWIN
            GI.ZD_GetObjectBound(ID_TWIN, out BoundWidth, out BoundHeight);
            x = xGlass - 20;
            y = yGlass + BoundHeightGlass - (int)(BoundHeight * 0.95);
            GI.ZD_SetObjectXY(ID_TWIN, x, y, GI.ZD_DRAW_DEFERRED);

            // ID_BRUSH
            GI.ZD_GetObjectBound(ID_BRUSH, out BoundWidth, out BoundHeight);
            x = xGlass + (int)Math.Max((BoundWidthGlass - BoundWidth * 0.95) / 2, 0);
            y = yGlass + (int)Math.Max((BoundHeightGlass - BoundHeight * 1.10) / 2, 0);
            GI.ZD_SetObjectXY(ID_BRUSH, x, y, GI.ZD_DRAW_DEFERRED);

            // Request GDImage to update the control display to show the changes
            GI.ZI_UpdateWindow(hCtrl, false);
        }

        ////[System.Security.Permissions.PermissionSet(System.Security.Permissions.SecurityAction.Demand, Name = "FullTrust")]
        //protected override void WndProc(ref Message m)
        //{
        //    // Listen for operating system messages.
        //    switch (m.Msg)
        //    {
        //        //case WM_xxx:
        //        //     break;
        //    }
        //    base.WndProc(ref m);
        //}

    }

    //****************************************************
    // GDImage............................................
    class GDImage
    {

        [DllImport("gdimage.dll")]  // Monitor the Windows message flow using a callback to take control over the GDImage default procedure.
        public static extern void ZI_EventMessage(
        CallBack pCodePtr,  // Pointer to a MyCallback function (see remark below).
        int wMsg,           // The Windows message to filter.
        Boolean SetReset    // TRUE = enable, FALSE = disable callback for the specific message.
        );

        public static IntPtr Ctrl(IntPtr hParent, int ID)
        {
            return Api.GetDlgItem(hParent, ID);
        }

        public static IntPtr CreateControl(Control T, int ID)
        {
            System.IntPtr hCtrl;
            hCtrl = GI.ZI_CreateWindow(
                T.Parent.Handle,    // The Window/Dialog container
                T.Location.X,       // Horizontal location
                T.Location.Y,       // Vertical location
                T.Size.Width,       // Width
                T.Size.Height,      // Height
                ID);                // Unique IDentifier
            return hCtrl;
        }

        public static IntPtr CreateZoomControl(Control T, int ID)
        {
            System.IntPtr hCtrl = CreateControl(T, ID);
            GI.ZI_SetProperty(hCtrl, GI.ZI_ZoomWindow, -1);
            return hCtrl;
        }

        public static IntPtr CreateThumbControl(Control T, int ID)
        {
            System.IntPtr hCtrl = CreateControl(T, ID);
            GI.ZI_SetProperty(hCtrl, GI.ZI_FitToWindow, GI.ZI_QualityDefault);
            return hCtrl;
        }

        public static void GradiantVert(IntPtr hCtrl, int TopColor, int BottomColor)
        {
            GI.ZI_SetProperty(hCtrl, GI.ZI_GradientTop, TopColor);
            GI.ZI_SetProperty(hCtrl, GI.ZI_GradientBottom, BottomColor);
        }

    }
}